#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 2018-09-18 - API de scikit-learn
# 
# Présentation de l'API de *scikit-learn* et implémentation d'un prédicteur fait maison. On utilise le jeu du Titanic qu'on peut récupérer sur [opendatasoft](https://public.opendatasoft.com/explore/dataset/titanic-passengers/?flg=fr) ou [awesome-public-datasets](https://github.com/awesomedata/awesome-public-datasets/tree/master/Datasets).

# In[1]:


import pandas
df = pandas.read_csv("titanic.csv/titanic.csv")
df.head(n=2)


# In[2]:


X, y = df[["Age", "Fare"]], df['Survived']


# In[3]:


from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y)


# In[4]:


from sklearn.linear_model import LogisticRegression
cls = LogisticRegression()
try:
    cls.fit(X_train, y_train)
except Exception as e:
    print(e)


# In[5]:


try:
    from sklearn.impute import SimpleImputer as Imputer
except ImportError:
    from sklearn.preprocessing import Imputer
imp = Imputer()
imp.fit(X_train)
X_train_nomiss = imp.transform(X_train)


# In[6]:


cls = LogisticRegression()
cls.fit(X_train_nomiss, y_train)


# In[7]:


cls.score(imp.transform(X_test), y_test)


# In[8]:


from sklearn.pipeline import Pipeline
pipe = Pipeline([("imputer", Imputer()), 
                 ("lr", LogisticRegression())])
pipe.fit(X_train, y_train)


# In[9]:


pipe.score(X_test, y_test)


# In[10]:


from sklearn.model_selection import GridSearchCV
grid = GridSearchCV(pipe, {"imputer__strategy": ['mean', 'most_frequent'],
                           "lr__max_iter": [5, 10, 50]})
grid.fit(X_train, y_train)


# In[11]:


res = pandas.DataFrame(grid.cv_results_)
col = [_ for _ in res.columns if 'param_' in _ or "test_score" in _]
res[col].T


# In[12]:


from sklearn.base import BaseEstimator, ClassifierMixin
import numpy

class MeanPredictor(BaseEstimator, ClassifierMixin):
    def __init__(self, alpha=0.5):
        self.alpha = alpha
        
    def fit(self, X, y):
        self.mean_ = int(self.alpha + numpy.mean(y))
        
    def predict(self, X):
        return numpy.array(list(self.mean_ for k in range(X.shape[0])))


# In[13]:


pipe_mean = Pipeline([('imputer', Imputer()), 
                      ('meanpredictor', MeanPredictor())])
pipe_mean.fit(X_train, y_train)


# In[14]:


pipe_mean.score(X_test, y_test)


# In[15]:


from sklearn.model_selection import GridSearchCV
grid = GridSearchCV(pipe_mean, {"imputer__strategy": ['mean', 'most_frequent'],
                                "meanpredictor__alpha": [0.2, 0.5, 0.8]})
grid.fit(X_train, y_train)


# In[16]:


res = pandas.DataFrame(grid.cv_results_)
col = [_ for _ in res.columns if 'param_' in _ or "test_score" in _]
res[col].T


# In[17]:


best = grid.best_estimator_


# In[18]:


import pickle
with open("model.pkl", "wb") as f:
    pickle.dump(best, f)


# In[19]:


with open("model.pkl", "rb") as f:
    model = pickle.load(f)


# In[20]:


model.predict(X_test) == best.predict(X_test)


# In[21]: